/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.1 Stauverlaufsanalyse
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.anastau.spez;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.ibv.anastau.spez.SpezifikationIT.TestVariante;
import de.bsvrz.sys.funclib.bitctrl.modell.AnmeldeException;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensendeException;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.objekte.StoerfallIndikator;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.onlinedaten.OdStoerfallZustand;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.zustaende.StoerfallSituation;

/**
 * Repräsentation eines Störfallindikators, der im Netz des Testverfahrens
 * verwendet wird und entsprehende Daten zur Verfügung stellt.
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public class TestIndikator {

	/**
	 * der verwendete Störfallindikator innerhalb der
	 * Datenverteilerkonfiguration.
	 */
	private final StoerfallIndikator indikator;

	/**
	 * die Zustande des Störfallindikators in den verschiedenen Phasen der
	 * Testdurchführung.
	 */
	private final List<StoerfallSituation> phasenSituationen = new ArrayList<>();

	/**
	 * Konstruktor.
	 *
	 * @param indikator
	 *            der verwendete Störfallindikator
	 * @throws AnmeldeException
	 *             die Anmeldung zur Bereitstellung der Testdaten ist
	 *             fehlgeschlagen
	 */
	public TestIndikator(final StoerfallIndikator indikator) throws AnmeldeException {
		this.indikator = indikator;
		indikator.getOnlineDatensatz(OdStoerfallZustand.class)
				.setQuelle(OdStoerfallZustand.Aspekte.StoerfallVerfahrenFuzzy.getAspekt(), true);
		indikator.getOnlineDatensatz(OdStoerfallZustand.class)
				.anmeldenSender(OdStoerfallZustand.Aspekte.StoerfallVerfahrenFuzzy.getAspekt());
	}

	/**
	 * liefert die Güte des Indikators für die übergebene Phase und die
	 * Testvariante.
	 *
	 * @param phase
	 *            die Nummer der aktuellen Phase
	 * @param variante
	 *            die verwendete Testvariante
	 * @return den Wert für die Güte
	 */
	protected double getGuete(final int phase, final TestVariante variante) {
		return 1.0;
	}

	/**
	 * liefert den verwendeten Störfallindikator.
	 *
	 * @return den Indikator
	 */
	public StoerfallIndikator getIndikator() {
		return indikator;
	}

	/**
	 * setzt die Daten für die entsprechende Testphase. Der allgemeine
	 * {@link TestIndikator} setzt den für die jeweilige Phase festgelegten
	 * Störfallzustand für des Verfahren Fuzzy.
	 *
	 * @param i
	 *            die Nummer der Phase
	 * @param variante
	 *            die aktuelle Testvariante
	 * @throws DatensendeException
	 *             die Daten konnten nicht über den Datenverteiler
	 *             bereitgestellt werden
	 */
	public void phase(final int i, final TestVariante variante) throws DatensendeException {
		if (phasenSituationen.size() > i) {
			final OdStoerfallZustand datenSatz = indikator.getOnlineDatensatz(OdStoerfallZustand.class);
			final OdStoerfallZustand.Daten daten = datenSatz.erzeugeDatum();
			daten.setSituation(phasenSituationen.get(i));
			daten.setGueteIndex(getGuete(i, variante));
			datenSatz.sendeDaten(OdStoerfallZustand.Aspekte.StoerfallVerfahrenFuzzy.getAspekt(), daten);
		}
	}

	/**
	 * setzt die Zustände des Störfallindikators für die verschiedenen Phasen
	 * der Testdurchführung.
	 *
	 * @param situationen
	 *            die Zustände als Array entsprechend der auszuführenden Phasen
	 */
	public void setzePhasen(final StoerfallSituation... situationen) {
		phasenSituationen.clear();
		for (final StoerfallSituation situation : situationen) {
			phasenSituationen.add(situation);
		}
	}
}
