/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.1 Stauverlaufsanalyse
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.anastau.spez;

import java.util.HashMap;
import java.util.Map;

import de.bsvrz.ibv.anastau.spez.SpezifikationIT.TestVariante;
import de.bsvrz.sys.funclib.bitctrl.modell.AnmeldeException;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensendeException;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.objekte.MessQuerschnitt;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.objekte.MessQuerschnittAllgemein;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.onlinedaten.OdVerkehrsDatenKurzZeitMq;

/**
 * Repräsentation eines Messquerschnitt innerhalb des für die Prüfung
 * vorgesehenen Netzes.
 *
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public abstract class TestMQ extends TestIndikator {

	/** die Menge aller angelegten MQ-Objekte. */
	private static final Map<MessQuerschnittAllgemein, TestMQ> MQS = new HashMap<>();

	/**
	 * liefert einen TestMQ, der den übergebenen messQuerschnitt innerhalb des
	 * Testnetztes repräsentiert. Wenn bisher keiner angelegt wurde, liefert die
	 * Funktion den Wert <code>null</code>.
	 *
	 * @param messQuerschnitt
	 *            der Messquerschnitt für den die Repräsentation im Testnetz
	 *            ermittelt werden soll
	 * @return den entsprechenden TestMQ oder <code>null</code>
	 */
	public static TestMQ getMq(final MessQuerschnittAllgemein messQuerschnitt) {
		return TestMQ.MQS.get(messQuerschnitt);
	}

	/** der Startzeitpunkt für die Testdurchführung. */
	private final long start;

	/**
	 * das Intervall mit dem die Testphasen ausgeführt werden sollen. Es
	 * entspricht dem Intervall, in dem die Stauverlaufsanalyse Stauobjekte
	 * bestimmt und prognostiziert.
	 */
	private final long intervall;

	/**
	 * Konstruktor.
	 *
	 * @param mq
	 *            der verwendete Messquerschnitt
	 * @param start
	 *            der Anfangszeitpunkt der Testdurchführung
	 * @param intervall
	 *            das Intervall der Phasen des Tests
	 * @throws AnmeldeException
	 *             eine geforderte Anmeldung beim Datenverteiler konnte nicht
	 *             erfolgreich ausgeführt werden
	 */
	public TestMQ(final MessQuerschnitt mq, final long start, final long intervall) throws AnmeldeException {
		super(mq);

		TestMQ.MQS.put(mq, this);

		this.start = start;
		this.intervall = intervall;

		getIndikator().getOnlineDatensatz(OdVerkehrsDatenKurzZeitMq.class)
				.setQuelle(OdVerkehrsDatenKurzZeitMq.Aspekte.Analyse.getAspekt(), true);
		getIndikator().getOnlineDatensatz(OdVerkehrsDatenKurzZeitMq.class)
				.anmeldenSender(OdVerkehrsDatenKurzZeitMq.Aspekte.Analyse.getAspekt());
	}

	/**
	 * liefert die Daten (Ganglinie) für den gegebenen Zeitpunkt.
	 *
	 * @param zeitpunkt
	 *            der Zeitpunkt
	 * @param nurLangfristig
	 *            nur langfristige Prognosedaten ermitteln
	 * @return die Menge der Daten
	 */
	public Double[] getDaten(final long zeitpunkt, final boolean nurLangfristig) {
		final int phase = (int) ((zeitpunkt - start) / intervall) + 1;
		return getPhasenDaten(phase, nurLangfristig);
	}

	/**
	 * liefert den Typ der Anfrage nach Ganglinien (NurLangfristig - true/false)
	 * für die gegebene Phase. Wenn innerhalb einer Phase in der
	 * Prüfspezifikation kein Typ definiert ist, wird der Wert <code>null</code>
	 * geliefert.
	 *
	 * @param variante
	 *            die Testvariante
	 * @param phase
	 *            die Phase
	 * @return der Anfragetyp oder <code>null</code>
	 */
	public Boolean getGlAnfrageTyp(final TestVariante variante, final int phase) {
		Boolean result = null;
		final Boolean[] typen = getGlAnfrageTypen();
		if ((variante.getPhasenAnzahl() - 1) > phase) {
			if ((typen != null) && (phase >= 0) && (phase < typen.length)) {
				result = typen[phase];
			}
		}
		return result;
	}

	/**
	 * liefert die für den MQ für die einzelnen Phasen definierten Anfragetypen,
	 * "nurlangfristig" - true/false oder <i>null</i>, wenn keine Prüfung
	 * erfolgen soll.
	 *
	 * @return die Typen als Feld
	 */
	protected abstract Boolean[] getGlAnfrageTypen();

	@Override
	public MessQuerschnitt getIndikator() {
		return (MessQuerschnitt) super.getIndikator();
	}

	/**
	 * liefert die Daten (Ganglinie) für die angegebene Phase.
	 *
	 * @param phase
	 *            die Nummer der Phase
	 * @param nurLangfristig
	 *            nur langfristige Prognosedaten liefern
	 * @return die Menge der Daten
	 */
	public abstract Double[] getPhasenDaten(int phase, boolean nurLangfristig);

	@Override
	public void phase(final int i, final TestVariante variante) throws DatensendeException {

		final Double[] werte = getPhasenDaten(i, false);

		if (werte.length > 0) {
			final OdVerkehrsDatenKurzZeitMq datenSatz = getIndikator()
					.getOnlineDatensatz(OdVerkehrsDatenKurzZeitMq.class);
			final OdVerkehrsDatenKurzZeitMq.Daten daten = datenSatz.erzeugeDatum();
			daten.setWert(OdVerkehrsDatenKurzZeitMq.Daten.Werte.QB.name(), werte[0]);
			datenSatz.sendeDaten(OdVerkehrsDatenKurzZeitMq.Aspekte.Analyse.getAspekt(), daten);
		}
		super.phase(i, variante);
	}
}
